@@ -1,24 +1,85 @@
 package Encode::Wechsler;
 use strict;
 use warnings FATAL => 'all';
-our $VERSION = '0.02';
-
-use Data::Dumper;
+our $VERSION = '0.03';
 
 my $i = 0;
-#{0, 1, 2, ..., 8, 9, a, b, ..., w} correspond to the bitstrings {'00000', '00001', '00010', ..., '11111'}.
+# {0, 1, 2, ..., 8, 9, a, b, ..., v} correspond to the bitstrings {'00000', '00001', '00010', ..., '11111'}.
 our %bits = map { $_ => sprintf("%05d", unpack( 'B32', pack( 'N', $i++ ) ) ) } 0 .. 9, 'a' .. 'v';
+our %stib = reverse %bits;
 
 $i = 4;
+# We use the characters 'w' and 'x' to abbreviate '00' and '000'
 # the symbols {'y0', 'y1', y2', ..., 'yx', 'yy', 'yz'} correspond to runs of between 4 and 39 consecutive '0's.
 our %zero = map { 'y' . $_ => 0 x $i++ } 0 .. 9, 'a' .. 'z';
+$zero{w} = '00';
+$zero{x} = '000';
+#our %orez = reverse %zero;
 
 sub new {
     my $self = shift;
     return bless {@_}, $self;
 }
 
-# x[spq][0-9]+_[0-9a-z]+
+
+sub encode {
+    my $self   = shift;
+    my %args   = @_;
+    my $thingy = $args{board};
+
+    # multi-line string
+    $thingy = [ split "\n", $thingy ] unless ref( $thingy );
+
+    # array of strings
+    $thingy = [ map [ split //, $_ ], @$thingy ] unless ref( $thingy->[0] );
+
+    # all become 2D array of true or false values
+    $thingy = [ map [ map { $_ eq '.' ? 0 : 1 } @$_ ], @$thingy ];
+
+=for later
+    my @chunks;
+    my $start_pruning;
+    for (my $i = 0; $i < @$thingy; $i += 5) {
+        # all zeros are ok until we found our first non all zeros
+        # then prune all zeros that are "trailing"
+        my @chunk;
+        for (@$thingy[ $i .. $i + 4 ]) {
+            $start_pruning ||= int( join '', @$_ );
+            if ($start_pruning) {
+                push @chunk, $_ if ref($_) && int( join '', @$_ );
+            } else {
+                push @chunk, $_ if ref($_);
+            }
+        }
+        push @chunks, [@chunk] if @chunk;
+    }
+=cut
+    my @chunks;
+    for (my $i = 0; $i < @$thingy; $i += 5) {
+        my @chunk = map { 
+            ref( $_ ) && int( join '', @$_ ) ? $_ : () 
+        } @$thingy[ $i .. $i + 4 ];
+        push @chunks, [@chunk];
+    }
+
+    @chunks = map _transpose( $_ ), @chunks;
+
+    #TODO: detect consecutive zero runs
+    my @bits;
+    for my $chunk (@chunks) {
+        push @bits, join '',
+            map { $stib{$_} }
+            map { sprintf '%05d', join '', reverse @$_ } @$chunk
+        ;
+    }
+
+    # this could be removed by detecting consecutive zero runs
+    my $str = join 'z', @bits;
+    $str =~ s/0*$//;
+
+    return $str;
+}
+
 sub decode {
     my ($self,$code) = @_;
 
@@ -30,10 +91,7 @@ sub decode {
 
     $self->{max} = 0;
     $format = join 'z', map {
-        # We use the characters 'w' and 'x' to abbreviate '00' and '000', respectively.
-        s/w/00/g;
-        s/x/000/g;
-        s/(y.)/$zero{$1}/g;
+        s/(w|x|y.)/$zero{$1}/g;
         $self->{max} = length($_) if length($_) > $self->{max};
         $_;
     } split 'z', $format;
@@ -95,6 +153,16 @@ sub _sum {
     return $s;
 }
 
+# credit: Math::Matrix
+sub _transpose {
+    my $data = shift;
+    my @trans;
+    for my $i (0 .. $#{ $data->[0] }) {
+        push @trans, [ map $_->[$i], @$data ]
+    }
+    return \@trans;
+}
+
 1;
 
 __END__
@@ -110,9 +178,9 @@ Object oriented interface:
 
   use Encode::Wechsler;
 
-  my $wechsler = Encode::Wechsler->new( pad => 1 );
-  my @array  = decode->( 'xp3_0ggmligkcz32w46' );
-  my $string = scalar decode->( 'xp3_0ggmligkcz32w46' );
+  my $wechsler = Encode::Wechsler->new;
+  my @array  = $wechsler->decode( 'xp3_0ggmligkcz32w46' );
+  my $string = scalar $wechsler->decode( 'xp3_0ggmligkcz32w46' );
 
 =head1 DESCRIPTION
 
@@ -153,32 +221,73 @@ Some codes require additional padding in order to sustain properly.
 
 =over 4
 
+=item C<encode( $string or $array_ref )>
+
+Not currently implemented.
+
 =item C<decode( $string )>
 
 Attempts to decode the string argument. Returns a list of array references (2d array)
 when called in list context:
 
-  my @array  = decode->( 'xp3_0ggmligkcz32w46' );
+  my @array  = $wechsler->decode( 'xp3_0ggmligkcz32w46' );
 
 Or returns a text representation of the game board in scalar context:
 
-  my $string = scalar decode->( 'xp3_0ggmligkcz32w46' );
+  my $string = scalar $wechsler->decode( 'xp3_0ggmligkcz32w46' );
 
 Throws exception if code is invalid.
 
 =back
 
-=head1 AUTHOR
+=head1 BUGS AND LIMITATIONS
 
-Jeff Anderson, C<< <jeffa at cpan.org> >>
+Please report any bugs or feature requests to either
+
+=over 4
+
+=item * Email: C<bug-encode-wechsler at rt.cpan.org>
+
+=item * Web: L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Encode-Wechsler>
+
+=back
+
+I will be notified, and then you'll automatically be notified of progress
+on your bug as I make changes.
+
+=head1 SUPPORT
+
+You can find documentation for this module with the perldoc command.
+
+    perldoc Encode::Wechsler
+
+The Github project is L<https://github.com/jeffa/Encode-Wechsler>
+
+You can also look for information at:
+
+=over 4
+
+=item * RT: CPAN's request tracker (report bugs here) L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Encode-Wechsler>
+
+=item * AnnoCPAN: Annotated CPAN documentation L<http://annocpan.org/dist/Encode-Wechsler>
+
+=item * CPAN Ratings L<http://cpanratings.perl.org/d/Encode-Wechsler>
+
+=item * Search CPAN L<http://search.cpan.org/dist/Encode-Wechsler/>
+
+=back
 
 =head1 ACKNOWLEDGEMENTS
 
 Pattern notion developed by Allan Wechsler 1992. L<https://catagolue.appspot.com/help/wechsler.txt>
 
+=head1 AUTHOR
+
+Jeff Anderson, C<< <jeffa at cpan.org> >>
+
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2016 Jeff Anderson.
+Copyright 2017 Jeff Anderson.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the the Artistic License (2.0). You may obtain a
